/*
   Copyright (C) 2011 PathScale Inc. All Rights Reserved.
*/

#include "initv_block_builder.h"
#include "symtab_defs.h"
#include "irbdata.h"


initv_block_builder::initv_block_builder():
block_idx_(New_INITV()),
last_idx_(INITV_IDX_ZERO),
size_(0)
{
    INITV_Init_Block(block_idx_, INITV_IDX_ZERO);
}


initv_block_builder::initv_block_builder(INITV_IDX block_idx):
block_idx_(block_idx),
last_idx_(INITV_blk(block_idx_)),
size_(0)
{
    FmtAssert(INITV_kind(block_idx) == INITVKIND_BLOCK, ("Invalid block initv"));

    if (last_idx_ == INITV_IDX_ZERO)
        return;

    size_ = 1;
    while (INITV_next(last_idx_) != INITV_IDX_ZERO) {
        ++size_;
        last_idx_ = INITV_next(last_idx_);
    }
}


void initv_block_builder::add_initv(INITV_IDX iv_idx) {
    if (last_idx_ == INITV_IDX_ZERO) {
        Set_INITV_blk(block_idx_, iv_idx);
    } else {
        Set_INITV_next(last_idx_, iv_idx);
    }

    last_idx_ = iv_idx;
    ++size_;
}


void initv_block_builder::add_symoff(const ST & st, INT32 ofst) {
    INITV_IDX iv_idx = New_INITV();
    INITV_Set_SYMOFF(Initv_Table[iv_idx], 1, ST_st_idx(st), ofst);
    add_initv(iv_idx);
}


void initv_block_builder::add_zero(TYPE_ID mtype) {
    INITV_IDX iv_idx = New_INITV();
    INITV_Set_ZERO(Initv_Table[iv_idx], mtype, 1);
    add_initv(iv_idx);
}


void initv_block_builder::add_one(TYPE_ID mtype) {
    INITV_IDX iv_idx = New_INITV();
    INITV_Set_ONE(Initv_Table[iv_idx], mtype, 1);
    add_initv(iv_idx);
}


void initv_block_builder::add_val(TCON_IDX val) {
    INITV_IDX iv_idx = New_INITV();
    INITV_Set_VAL(Initv_Table[iv_idx], val, 1);
    add_initv(iv_idx);
}


void initv_block_builder::add_pad(UINT32 bytes) {
    INITV_IDX iv_idx = New_INITV();
    INITV_Set_PAD(Initv_Table[iv_idx], bytes);
    add_initv(iv_idx);
}


void initv_block_builder::add_symdiff(LABEL_IDX s1, const ST & s2) {
    INITV_IDX iv_idx = New_INITV();
    INITV_Set_SYMDIFF(Initv_Table[iv_idx], 1, s1, ST_st_idx(s2), FALSE);
    add_initv(iv_idx);
}


void initv_block_builder::add_symdiff16(LABEL_IDX s1, const ST & s2) {
    INITV_IDX iv_idx = New_INITV();
    INITV_Set_SYMDIFF(Initv_Table[iv_idx], 1, s1, ST_st_idx(s2), TRUE);
    add_initv(iv_idx);
}


initv_block_builder::const_iterator & initv_block_builder::const_iterator::operator++() {
    FmtAssert(iv_idx_ != INITV_IDX_ZERO, ("Invalid block iterator"));
    iv_idx_ = INITV_next(iv_idx_);
    return *this;
}


initv_block_builder::const_iterator initv_block_builder::begin() const {
    return const_iterator(INITV_blk(block_idx_));
}


