/*
   Copyright (C) 2011 PathScale Inc. All Rights Reserved.
*/


#ifndef initv_block_builder_INCLUDED
#define initv_block_builder_INCLUDED

#include "symtab_idx.h"
#include "mtypes.h"
#include <iterator>


class initv_block_builder {
public:
    initv_block_builder();
    initv_block_builder(INITV_IDX block_idx);

    INITV_IDX result() const {
        return block_idx_;
    }

    void add_initv(INITV_IDX iv_idx);
    void add_symoff(const ST & st, INT32 ofst = 0);
    void add_zero(TYPE_ID mtype);
    void add_one(TYPE_ID mtype);
    void add_val(TCON_IDX val);
    void add_pad(UINT32 bytes);
    void add_symdiff(LABEL_IDX s1, const ST & s2);
    void add_symdiff16(LABEL_IDX s1, const ST & s2);

    std::size_t size() const {
        return size_;
    }

    bool empty() const {
        return size_ == 0;
    }


    class const_iterator: public std::iterator<std::input_iterator_tag, INITV_IDX> {
    public:
        const_iterator(INITV_IDX idx = INITV_IDX_ZERO):
            iv_idx_(idx) {}

        INITV_IDX operator*() const {
            return iv_idx_;
        }

        const_iterator & operator++();

        bool operator==(const const_iterator & it) const {
            return iv_idx_ == it.iv_idx_;
        }

        bool operator!=(const const_iterator & it) const {
            return !(*this == it);
        }

    private:
        INITV_IDX iv_idx_;
    };

    const_iterator begin() const;

    const_iterator end() const {
        return const_iterator();
    }

    INITV_IDX at(int index) const {
        int ind = 0;
        for (const_iterator it = begin(), e = end(); it != e; ++it) {
            if (ind == index)
                return *it;

            ++ind;
        }

        FmtAssert(FALSE, ("Invalid block index"));
        return INITV_IDX_ZERO;
    }

private:
    INITV_IDX block_idx_;
    INITV_IDX last_idx_;
    std::size_t size_;
};

#endif // initv_block_builder_INCLUDED

